import cv2
import numpy as np
from enum import Enum , unique , auto
import torch
import sys
sys.path.append('DetectHelper')
from models.yolo import attempt_load
from utils.general import non_max_suppression , scale_coords , xyxy2xywh
from utils.datasets import letterbox
from utils.plots import plot_one_box
from utils.torch_utils import select_device

@unique
class DetectImgType(Enum):
    GrayWithRect = auto()
    ColorWithRect = auto() 
    GrayWithoutBackGround = auto()
    ColorWithoutBackGround = auto()
    DrawTemplate = auto()
    DrawAllRects = auto()


class DetectHelper:
    m_threshold = 0.6 #过滤阈值
    m_targetName = 0 #选取的检测到的类名
    m_rectColor=[255,255,255]
    m_imgsz = 640
    def __init__(self , threshold: float,imgsz: int  ) :
        self.m_threshold = threshold
        self.m_imgsz = imgsz
    
    #读取模型
    def loadModule(self , modulePath:str ,DEVICE:str) : 
        self.m_device = select_device(DEVICE)
        self.m_module = attempt_load(modulePath,map_location=self.m_device)
        # self.m_module.half()  half仅能在使用gpu时候打开
        self.m_stride = int(self.m_module.stride.max()) 
        self.m_names = self.m_module.module.names if hasattr(self.m_module, 'module') else self.m_module.names  #类别名称
        self.m_module.eval()
    
    def setThreshold(self , threshold: float) :
        self.m_threshold = threshold
        return 
    
    #设置过滤类名
    def setClassName(self ,cln: str) : 
        for i , v in enumerate(self.m_names) : 
            if v == cln :
                self.m_targetName = i
                return
        return 
    
    #设置绘图颜色
    def setRectColor(self , r: int , g:int , b:int) :
        self.m_rectColor = [r , g, b]
        return
    
    #对图像预处理：先校验img的形状->letterbox改变大小->transpos改变通道
    def preProccessed(self ,  img0: cv2.Mat):
         # Padded resize
        img = letterbox(img0, self.m_imgsz , stride=self.m_stride)[0]
        # Convert
        img = img[:, :, ::-1].transpose(2, 0, 1)  # BGR to RGB, to 3x416x416
        img = np.ascontiguousarray(img)
        return  img
    
    #在源图像对应的位置绘制矩形框
    def drawRect(self,img0: cv2.Mat , xyxy , cls:int, conf:float):
        label = f'{self.m_names[cls]} {conf:.2f}'
        plot_one_box(xyxy, img0, label=label, color= self.m_rectColor, line_thickness=1)  #标记框
        return img0
    
    def backProccessed(self ,sourceImg:cv2.Mat ,xyxy , cls:int, conf:float ,  detectType: DetectImgType) :
    #         GrayWithRect = auto()
    # ColorWithRect = auto() 
    # GrayWithoutBackGround = auto()
    # ColorWithoutBackGround = auto()
    # DrawTemplate = auto()
    # DrawAllRects = auto()
    # 由于raft限制，只能接受3通道的输入，这里灰度处理为同样数据复制3个通道，因此效果不太行
        if detectType == DetectImgType.GrayWithRect.value :
            img = cv2.cvtColor(sourceImg, cv2.COLOR_BGR2GRAY)
            retImg = img
            # retImg = self.drawRect(img , xyxy , cls , conf)
            retImg = np.expand_dims(retImg , 2).repeat(3,axis=2) #灰度通道处理
            return retImg 
        if detectType == DetectImgType.ColorWithRect.value :
            retImg = sourceImg
            # retImg = self.drawRect(sourceImg , xyxy , cls , conf)
            return retImg
        if detectType == DetectImgType.GrayWithoutBackGround.value :
            retImg = cv2.cvtColor(sourceImg, cv2.COLOR_BGR2GRAY)
            
            for i in range(0 , retImg.shape[0]-1) :
                for j in range(0 , retImg.shape[1]-1):
                    if (i > xyxy[3] or i < xyxy[1]) or (j > xyxy[2] or j < xyxy[0]) :
                        retImg[i][j] = 0
            retImg = np.expand_dims(retImg , 2).repeat(3,axis=2) #灰度通道处理
            return retImg
        if detectType == DetectImgType.ColorWithoutBackGround.value :
            for i in range(0 , sourceImg.shape[0]-1) :
                for j in range(0 , sourceImg.shape[1]-1):
                    if (i > xyxy[3] or i < xyxy[1]) or (j > xyxy[2] or j < xyxy[0]) :
                        sourceImg[i][j] = sourceImg[i][j] *0
            return sourceImg

        
        # img = self.drawRect(sourceImg , xyxy , cls , conf )
        return sourceImg 
    
    def Detect(self, img0 : cv2.Mat , detectType : DetectImgType ):
        img = self.preProccessed(img0)
        img = torch.from_numpy(img).to(self.m_device)
        img = img.float()   # uint8 to fp16/32
        img /= 255.0  # 0 - 255 to 0.0 - 1.0
        if img.ndimension() == 3:  #修改为符合batch
            img = img.unsqueeze(0)


        with torch.no_grad():
            pred = self.m_module(img , augment = True)[0]
            pred = non_max_suppression(pred,  agnostic=True)

        picThread = self.m_threshold
        
        retxyxy = None
        #单幅图中的所有对象信息
        im0 = img0.copy()
        for i, det in enumerate(pred):  # detections per image
        
            gn = torch.tensor(im0.shape)[[1, 0, 1, 0]]  # normalization gain whwh
            if len(det):
                # Rescale boxes from img_size to im0 size
                det[:, :4] = scale_coords(img.shape[2:], det[:, :4], im0.shape).round()
                #  对推理结果进行解析
                for *xyxy, conf, cls in reversed(det):
                    #xyxy:对角顶点，  cls:类别名
                    if conf > picThread and int(cls) == self.m_targetName:  # Write to file
                        #遍历结果，寻找最符合标准的花的结果
                        picThread = conf
                        retxyxy = xyxy
                        # retxywh = (xyxy2xywh(torch.tensor(xyxy).view(1, 4)) / gn).view(-1).tolist()  # normalized xywh
                    
                    self.drawRect(im0 , xyxy, int(cls) , conf)
        cv2.imwrite("demo_output/alldetect.png" , im0)
        # exit()
        if retxyxy == None :
            return None , None
        
        retImg = self.backProccessed(img0 , retxyxy , self.m_targetName , picThread ,detectType )
        return  retImg ,  retxyxy